@@ -14,23 +14,29 @@ from threading import Lock
 from time import strptime
 
 from . import errors
+from . import proxy
 from .auth import Auth
 from .auth_default import AuthByDefault
 from .auth_keypair import AuthByKeyPair
 from .auth_oauth import AuthByOAuth
 from .auth_okta import AuthByOkta
 from .auth_webbrowser import AuthByWebBrowser
-from .chunk_downloader import SnowflakeChunkDownloader
-from .compat import (TO_UNICODE, IS_OLD_PYTHON, urlencode, PY2, PY_ISSUE_23517)
+from .chunk_downloader import (
+    SnowflakeChunkDownloader,
+    DEFAULT_CLIENT_PREFETCH_THREADS,
+    MAX_CLIENT_PREFETCH_THREADS)
+from .compat import (
+    TO_UNICODE, IS_OLD_PYTHON, urlencode, PY2, PY_ISSUE_23517, IS_WINDOWS)
 from .constants import (
     PARAMETER_AUTOCOMMIT,
     PARAMETER_CLIENT_SESSION_KEEP_ALIVE_HEARTBEAT_FREQUENCY,
     PARAMETER_CLIENT_SESSION_KEEP_ALIVE,
     PARAMETER_CLIENT_TELEMETRY_ENABLED,
     PARAMETER_TIMEZONE,
+    PARAMETER_SERVICE_NAME,
+    PARAMETER_CLIENT_STORE_TEMPORARY_CREDENTIAL,
+    PARAMETER_CLIENT_PREFETCH_THREADS,
 )
-from .converter import SnowflakeConverter
-from .converter_issue23517 import SnowflakeConverterIssue23517
 from .cursor import SnowflakeCursor
 from .errorcode import (ER_CONNECTION_IS_CLOSED,
                         ER_NO_ACCOUNT_NAME, ER_OLD_PYTHON, ER_NO_USER,
@@ -60,6 +66,16 @@ from .time_util import (
     HeartBeatTimer, get_time_millis)
 from .util_text import split_statements, construct_hostname
 
+
+def DefaultConverterClass():
+    if PY_ISSUE_23517 or IS_WINDOWS:
+        from .converter_issue23517 import SnowflakeConverterIssue23517
+        return SnowflakeConverterIssue23517
+    else:
+        from .converter import SnowflakeConverter
+        return SnowflakeConverter
+
+
 SUPPORTED_PARAMSTYLES = {
     u"qmark",
     u'numeric',
@@ -104,16 +120,18 @@ DEFAULT_CONFIGURATION = {
     u'autocommit': None,  # snowflake
     u'client_session_keep_alive': False,  # snowflake
     u'client_session_keep_alive_heartbeat_frequency': None,  # snowflake
+    u'client_prefetch_threads': 4,  # snowflake
     u'numpy': False,  # snowflake
     u'ocsp_response_cache_filename': None,  # snowflake internal
-    u'converter_class':
-        SnowflakeConverter if not PY_ISSUE_23517 else SnowflakeConverterIssue23517,
+    u'converter_class': DefaultConverterClass(),
     u'chunk_downloader_class': SnowflakeChunkDownloader,  # snowflake internal
     u'validate_default_parameters': False,  # snowflake
     u'probe_connection': False,  # snowflake
     u'paramstyle': None,  # standard/snowflake
     u'timezone': None,  # snowflake
     u'consent_cache_id_token': True,  # snowflake
+    u'service_name': None,  # snowflake,
+    u'support_negative_year': True  # snowflake
 }
 
 APPLICATION_RE = re.compile(r'[\w\d_]+')
@@ -318,6 +336,22 @@ class SnowflakeConnection(object):
         self._client_session_keep_alive_heartbeat_frequency = value
         self._validate_client_session_keep_alive_heartbeat_frequency()
 
+    @property
+    def client_prefetch_threads(self):
+        u"""
+        Number of threads to download the result set
+        """
+        return self._client_prefetch_threads if \
+            self._client_prefetch_threads else DEFAULT_CLIENT_PREFETCH_THREADS
+
+    @client_prefetch_threads.setter
+    def client_prefetch_threads(self, value):
+        """
+        Number of threads to download the result set
+        """
+        self._client_prefetch_threads = value
+        self._validate_client_prefetch_threads()
+
     @property
     def rest(self):
         u"""
@@ -385,6 +419,14 @@ class SnowflakeConnection(object):
     def telemetry_enabled(self, value):
         self._telemetry_enabled = True if value else False
 
+    @property
+    def service_name(self):
+        return self._service_name
+
+    @service_name.setter
+    def service_name(self, value):
+        self._service_name = value
+
     def connect(self, **kwargs):
         u"""
         Connects to the database
@@ -483,7 +525,8 @@ class SnowflakeConnection(object):
 
     def execute_string(self, sql_text,
                        remove_comments=False,
-                       return_cursors=True):
+                       return_cursors=True,
+                       **kwargs):
         """
         Executes a SQL text including multiple statements.
         This is a non-standard convenient method.
@@ -499,7 +542,7 @@ class SnowflakeConnection(object):
             cur = self.cursor()
             if return_cursors:
                 ret.append(cur)
-            cur.execute(sql, _is_put_get=is_put_or_get)
+            cur.execute(sql, _is_put_get=is_put_or_get, **kwargs)
         return ret
 
     def execute_stream(self, stream,
@@ -524,26 +567,22 @@ class SnowflakeConnection(object):
         Opens a new network connection
         """
         self.converter = self._converter_class(
-            use_sfbinaryformat=False,
-            use_numpy=self._numpy)
+            use_numpy=self._numpy,
+            support_negative_year=self._support_negative_year)
+
+        proxy.set_proxies(
+            self.proxy_host, self.proxy_port, self.proxy_user,
+            self.proxy_password)
 
         self._rest = SnowflakeRestful(
             host=self.host,
             port=self.port,
-            proxy_host=self.proxy_host,
-            proxy_port=self.proxy_port,
-            proxy_user=self.proxy_user,
-            proxy_password=self.proxy_password,
             protocol=self._protocol,
             inject_client_pause=self._inject_client_pause,
             connection=self)
-        logger.debug(u'REST API object was created: %s:%s, proxy=%s:%s, '
-                     u'proxy_user=%s',
+        logger.debug(u'REST API object was created: %s:%s',
                      self.host,
-                     self.port,
-                     self.proxy_host,
-                     self.proxy_port,
-                     self.proxy_user)
+                     self.port)
 
         if self.host.endswith(u".privatelink.snowflakecomputing.com"):
             ocsp_cache_server = \
@@ -585,9 +624,14 @@ class SnowflakeConnection(object):
                 PARAMETER_CLIENT_SESSION_KEEP_ALIVE_HEARTBEAT_FREQUENCY] = \
                 self._validate_client_session_keep_alive_heartbeat_frequency()
 
+        if self.client_prefetch_threads:
+            self._session_parameters[PARAMETER_CLIENT_PREFETCH_THREADS] = \
+                self._validate_client_prefetch_threads()
+
         if self._authenticator == EXTERNAL_BROWSER_AUTHENTICATOR:
             # enable storing temporary credential in a file
-            self._session_parameters['CLIENT_STORE_TEMPORARY_CREDENTIAL'] = True
+            self._session_parameters[
+                PARAMETER_CLIENT_STORE_TEMPORARY_CREDENTIAL] = True
 
         auth = Auth(self.rest)
         if not auth.read_temporary_credential(
@@ -795,6 +839,7 @@ class SnowflakeConnection(object):
     def __authenticate(self, auth_instance):
         auth_instance.authenticate(
             authenticator=self._authenticator,
+            service_name=self.service_name,
             account=self.account,
             user=self.user,
             password=self._password,
@@ -998,6 +1043,15 @@ class SnowflakeConnection(object):
             self.client_session_keep_alive_heartbeat_frequency)
         return self.client_session_keep_alive_heartbeat_frequency
 
+    def _validate_client_prefetch_threads(self):
+        if self.client_prefetch_threads <= 0:
+            self._client_prefetch_threads = 1
+        elif self.client_prefetch_threads > MAX_CLIENT_PREFETCH_THREADS:
+            self._client_prefetch_threads = MAX_CLIENT_PREFETCH_THREADS
+        self._client_prefetch_threads = int(
+            self.client_prefetch_threads)
+        return self.client_prefetch_threads
+
     def _set_parameters(self, ret, session_parameters):
         """
         Set session parameters
@@ -1018,6 +1072,10 @@ class SnowflakeConnection(object):
             elif PARAMETER_CLIENT_SESSION_KEEP_ALIVE_HEARTBEAT_FREQUENCY == \
                     name:
                 self.client_session_keep_alive_heartbeat_frequency = value
+            elif PARAMETER_SERVICE_NAME == name:
+                self.service_name = value
+            elif PARAMETER_CLIENT_PREFETCH_THREADS == name:
+                self.client_prefetch_threads = value
 
     def __enter__(self):
         u"""
