@@ -10,7 +10,6 @@ import itertools
 import json
 import logging
 import platform
-import random
 import sys
 import time
 import uuid
@@ -27,7 +26,6 @@ from botocore.vendored.requests.packages.urllib3.exceptions import (
     ProtocolError, ReadTimeoutError)
 
 from snowflake.connector.time_util import get_time_millis
-from . import proxy
 from . import ssl_wrap_socket
 from .compat import (
     PY2,
@@ -35,7 +33,12 @@ from .compat import (
     FORBIDDEN, BAD_GATEWAY, REQUEST_TIMEOUT,
     UNAUTHORIZED, INTERNAL_SERVER_ERROR, OK, BadStatusLine)
 from .compat import (TO_UNICODE, urlencode, urlparse)
-from .compat import proxy_bypass
+from .constants import (
+    HTTP_HEADER_CONTENT_TYPE,
+    HTTP_HEADER_ACCEPT,
+    HTTP_HEADER_USER_AGENT,
+    HTTP_HEADER_SERVICE_NAME,
+)
 from .errorcode import (ER_FAILED_TO_CONNECT_TO_DB, ER_CONNECTION_IS_CLOSED,
                         ER_FAILED_TO_REQUEST, ER_FAILED_TO_RENEW_SESSION)
 from .errors import (Error, OperationalError, DatabaseError, ProgrammingError,
@@ -47,9 +50,10 @@ from .gzip_decoder import decompress_raw_data
 from .sqlstate import (SQLSTATE_CONNECTION_NOT_EXISTS,
                        SQLSTATE_CONNECTION_WAS_NOT_ESTABLISHED,
                        SQLSTATE_CONNECTION_REJECTED)
-from .ssl_wrap_socket import (set_proxies)
 from .time_util import (
-    DEFAULT_MASTER_VALIDITY_IN_SECONDS)
+    DecorrelateJitterBackoff,
+    DEFAULT_MASTER_VALIDITY_IN_SECONDS
+)
 from .tool.probe_connection import probe_connection
 from .util_text import split_rows_from_stream
 from .version import VERSION
@@ -96,6 +100,7 @@ HEADER_SNOWFLAKE_TOKEN = u'Snowflake Token="{token}"'
 
 REQUEST_ID = u'requestId'
 REQUEST_GUID = u'request_guid'
+SNOWFLAKE_HOST_SUFFIX = u'.snowflakecomputing.com'
 
 SNOWFLAKE_CONNECTOR_VERSION = u'.'.join(TO_UNICODE(v) for v in VERSION[0:3])
 PYTHON_VERSION = u'.'.join(TO_UNICODE(v) for v in sys.version_info[:3])
@@ -203,19 +208,11 @@ class SnowflakeRestful(object):
     """
 
     def __init__(self, host=u'127.0.0.1', port=8080,
-                 proxy_host=None,
-                 proxy_port=None,
-                 proxy_user=None,
-                 proxy_password=None,
                  protocol=u'http',
                  inject_client_pause=0,
                  connection=None):
         self._host = host
         self._port = port
-        self._proxy_host = proxy_host
-        self._proxy_port = proxy_port
-        self._proxy_user = proxy_user
-        self._proxy_password = proxy_password
         self._protocol = protocol
         self._inject_client_pause = inject_client_pause
         self._connection = connection
@@ -230,14 +227,9 @@ class SnowflakeRestful(object):
         ssl_wrap_socket.FEATURE_OCSP_RESPONSE_CACHE_FILE_NAME = \
             self._connection and self._connection._ocsp_response_cache_filename
         #
-        proxy.PROXY_HOST = self._proxy_host
-        proxy.PROXY_PORT = self._proxy_port
-        proxy.PROXY_USER = self._proxy_user
-        proxy.PROXY_PASSWORD = self._proxy_password
 
         # This is to address the issue where requests hangs
         _ = 'dummy'.encode('idna').decode('utf-8')  # noqa
-        proxy_bypass('www.snowflake.net:443')
 
     @property
     def token(self):
@@ -267,11 +259,6 @@ class SnowflakeRestful(object):
     def id_token(self, value):
         self._id_token = value
 
-    @property
-    def id_token_password(self):
-        return self._id_token_password if hasattr(
-            self, u'_id_token_password') else None
-
     def close(self):
         if hasattr(self, u'_token'):
             del self._token
@@ -279,11 +266,9 @@ class SnowflakeRestful(object):
             del self._master_token
         if hasattr(self, u'_id_token'):
             del self._id_token
-        if hasattr(self, u'_id_token_password'):
-            del self._id_token_password
         sessions = list(self._active_sessions)
         if sessions:
-            logger.warning("Closing %s active sessions", len(sessions))
+            logger.debug("Closing %s active sessions", len(sessions))
         sessions.extend(self._idle_sessions)
         self._active_sessions.clear()
         self._idle_sessions.clear()
@@ -291,7 +276,7 @@ class SnowflakeRestful(object):
             try:
                 s.close()
             except Exception as e:
-                logger.warning("Session cleanup failed: %s", e)
+                logger.info("Session cleanup failed: %s", e)
 
     def request(self, url, body=None, method=u'post', client=u'sfsql',
                 _no_results=False, timeout=None, _include_retry_params=False):
@@ -315,10 +300,12 @@ class SnowflakeRestful(object):
             timeout = self._connection.network_timeout
 
         headers = {
-            u'Content-Type': CONTENT_TYPE_APPLICATION_JSON,
-            u"accept": accept_type,
-            u"User-Agent": PYTHON_CONNECTOR_USER_AGENT,
+            HTTP_HEADER_CONTENT_TYPE: CONTENT_TYPE_APPLICATION_JSON,
+            HTTP_HEADER_ACCEPT: accept_type,
+            HTTP_HEADER_USER_AGENT: PYTHON_CONNECTOR_USER_AGENT,
         }
+        if self._connection.service_name:
+            headers[HTTP_HEADER_SERVICE_NAME] = self._connection.service_name
         if method == u'post':
             return self._post_request(
                 url, headers, json.dumps(body),
@@ -331,7 +318,7 @@ class SnowflakeRestful(object):
 
     def update_tokens(self, session_token, master_token,
                       master_validity_in_seconds=None,
-                      id_token=None, id_token_password=None):
+                      id_token=None):
         """
         Update session and master tokens and optionally temporary credential
         """
@@ -339,7 +326,6 @@ class SnowflakeRestful(object):
             self._token = session_token
             self._master_token = master_token
             self._id_token = id_token
-            self._id_token_password = id_token_password
             self._master_validity_in_seconds = master_validity_in_seconds
 
     def _renew_session(self):
@@ -366,10 +352,12 @@ class SnowflakeRestful(object):
             u'****' if self.master_token else None,
             u'****' if self.id_token else None)
         headers = {
-            u'Content-Type': CONTENT_TYPE_APPLICATION_JSON,
-            u"accept": CONTENT_TYPE_APPLICATION_JSON,
-            u"User-Agent": PYTHON_CONNECTOR_USER_AGENT,
+            HTTP_HEADER_CONTENT_TYPE: CONTENT_TYPE_APPLICATION_JSON,
+            HTTP_HEADER_ACCEPT: CONTENT_TYPE_APPLICATION_JSON,
+            HTTP_HEADER_USER_AGENT: PYTHON_CONNECTOR_USER_AGENT,
         }
+        if self._connection.service_name:
+            headers[HTTP_HEADER_SERVICE_NAME] = self._connection.service_name
         request_id = TO_UNICODE(uuid.uuid4())
         logger.debug(u'request_id: %s', request_id)
         url = u'/session/token-request?' + urlencode({
@@ -400,8 +388,7 @@ class SnowflakeRestful(object):
                 ret[u'data'].get(u'masterToken'),
                 master_validity_in_seconds=ret[u'data'].get(
                     u'masterValidityInSeconds'),
-                id_token=self.id_token,
-                id_token_password=self.id_token_password)
+                id_token=self.id_token)
             logger.debug(u'updating session completed')
             ret[UPDATED_BY_ID_TOKEN] = request_type == REQUEST_TYPE_ISSUE
             return ret
@@ -433,10 +420,12 @@ class SnowflakeRestful(object):
 
     def _heartbeat(self):
         headers = {
-            u'Content-Type': CONTENT_TYPE_APPLICATION_JSON,
-            u"accept": CONTENT_TYPE_APPLICATION_JSON,
-            u"User-Agent": PYTHON_CONNECTOR_USER_AGENT,
+            HTTP_HEADER_CONTENT_TYPE: CONTENT_TYPE_APPLICATION_JSON,
+            HTTP_HEADER_ACCEPT: CONTENT_TYPE_APPLICATION_JSON,
+            HTTP_HEADER_USER_AGENT: PYTHON_CONNECTOR_USER_AGENT,
         }
+        if self._connection.service_name:
+            headers[HTTP_HEADER_SERVICE_NAME] = self._connection.service_name
         request_id = TO_UNICODE(uuid.uuid4())
         logger.debug(u'request_id: %s', request_id)
         url = u'/session/heartbeat?' + urlencode({
@@ -464,10 +453,13 @@ class SnowflakeRestful(object):
 
         url = u'/session?' + urlencode({u'delete': u'true'})
         headers = {
-            u'Content-Type': CONTENT_TYPE_APPLICATION_JSON,
-            u"accept": CONTENT_TYPE_APPLICATION_JSON,
-            u"User-Agent": PYTHON_CONNECTOR_USER_AGENT,
+            HTTP_HEADER_CONTENT_TYPE: CONTENT_TYPE_APPLICATION_JSON,
+            HTTP_HEADER_ACCEPT: CONTENT_TYPE_APPLICATION_JSON,
+            HTTP_HEADER_USER_AGENT: PYTHON_CONNECTOR_USER_AGENT,
         }
+        if self._connection.service_name:
+            headers[HTTP_HEADER_SERVICE_NAME] = self._connection.service_name
+
         body = {}
         try:
             ret = self._post_request(
@@ -580,17 +572,6 @@ class SnowflakeRestful(object):
               **kwargs):
         """ Curried API request with session management. """
 
-        class DecorrelateJitterBackoff(object):
-            # Decorrelate Jitter backoff
-            # see:
-            # https://www.awsarchitectureblog.com/2015/03/backoff.html
-            def __init__(self, base, cap):
-                self._base = base
-                self._cap = cap
-
-            def next_sleep(self, _, sleep):
-                return min(self._cap, random.randint(self._base, sleep * 3))
-
         class RetryCtx(object):
             def __init__(self, timeout, _include_retry_params=False):
                 self.timeout = timeout
@@ -633,10 +614,13 @@ class SnowflakeRestful(object):
         """
         Add request_guid parameter for HTTP request tracing
         """
+        parsed_url = urlparse(full_url)
+        if not parsed_url.hostname.endswith(SNOWFLAKE_HOST_SUFFIX):
+            return full_url
         suffix = urlencode({
             REQUEST_GUID: TO_UNICODE(uuid.uuid4())
         })
-        sep = '&' if urlparse(full_url).query else '?'
+        sep = '&' if parsed_url.query else '?'
         # url has query string already, just add fields
         return full_url + sep + suffix
 
@@ -647,8 +631,6 @@ class SnowflakeRestful(object):
             **kwargs):
 
         conn = self._connection
-        proxies = set_proxies(self._proxy_host, self._proxy_port,
-                              self._proxy_user, self._proxy_password)
         logger.debug('remaining request timeout: %s, retry cnt: %s',
                      retry_ctx.timeout, retry_ctx.cnt + 1)
 
@@ -662,13 +644,12 @@ class SnowflakeRestful(object):
                 full_url=full_url,
                 headers=headers,
                 data=data,
-                proxies=proxies,
                 token=token,
                 **kwargs)
             if return_object is not None:
                 return return_object
             self._handle_unknown_error(
-                method, full_url, headers, data, conn, proxies)
+                method, full_url, headers, data, conn)
             return {}
         except RetryRequest as e:
             if no_retry:
@@ -716,7 +697,7 @@ class SnowflakeRestful(object):
             })
 
     def _handle_unknown_error(
-            self, method, full_url, headers, data, conn, proxies):
+            self, method, full_url, headers, data, conn):
         """
         Handle unknown error
         """
@@ -733,23 +714,20 @@ class SnowflakeRestful(object):
         logger.error(
             u'Failed to get the response. Hanging? '
             u'method: {method}, url: {url}, headers:{headers}, '
-            u'data: {data}, proxies: {proxies}'.format(
+            u'data: {data}'.format(
                 method=method,
                 url=full_url,
                 headers=headers,
                 data=data,
-                proxies=proxies
             )
         )
         Error.errorhandler_wrapper(
             conn, None, OperationalError,
             {
                 u'msg': u'Failed to get the response. Hanging? '
-                        u'method: {method}, url: {url}, '
-                        u'proxies: {proxies}'.format(
+                        u'method: {method}, url: {url}'.format(
                     method=method,
                     url=full_url,
-                    proxies=proxies
                 ),
                 u'errno': ER_FAILED_TO_REQUEST,
             })
@@ -757,7 +735,6 @@ class SnowflakeRestful(object):
     def _request_exec(
             self,
             session, method, full_url, headers, data,
-            proxies,
             token,
             catch_okta_unauthorized_error=False,
             is_raw_text=False,
@@ -791,7 +768,6 @@ class SnowflakeRestful(object):
             raw_ret = session.request(
                 method=method,
                 url=full_url,
-                proxies=proxies,
                 headers=headers,
                 data=input_data,
                 timeout=socket_timeout,
@@ -829,8 +805,7 @@ class SnowflakeRestful(object):
                         ret = raw_ret.json()
                     return ret
 
-                if is_retryable_http_code(
-                        raw_ret.status_code):
+                if is_retryable_http_code(raw_ret.status_code):
                     ex = STATUS_TO_EXCEPTION.get(
                         raw_ret.status_code, OtherHTTPRetryableError)
                     exi = ex(code=raw_ret.status_code)
