@@ -27,7 +27,7 @@ from pyasn1_modules import (rfc2459, rfc2437, rfc2560)
 
 from snowflake.connector.ocsp_snowflake import SnowflakeOCSP
 from .compat import (PY2)
-from .errorcode import (ER_INVALID_OCSP_RESPONSE)
+from .errorcode import (ER_INVALID_OCSP_RESPONSE, ER_INVALID_OCSP_RESPONSE_CODE)
 from .errors import (OperationalError)
 from .rfc6960 import (
     OCSPRequest,
@@ -39,6 +39,8 @@ from .rfc6960 import (
     BasicOCSPResponse,
     Version)
 
+from snowflake.connector.ssd_internal_keys import ret_wildcard_hkey
+
 logger = getLogger(__name__)
 
 
@@ -63,6 +65,8 @@ class SnowflakeOCSPPyasn1(SnowflakeOCSP):
         sha512WithRSAEncryption: SHA512,
     }
 
+    WILDCARD_CERTID = None
+
     @staticmethod
     def _get_pyasn1_version():
         with SnowflakeOCSPPyasn1.PYASN1_VERSION_LOCK:
@@ -78,6 +82,7 @@ class SnowflakeOCSPPyasn1(SnowflakeOCSP):
 
     def __init__(self, **kwargs):
         super(SnowflakeOCSPPyasn1, self).__init__(**kwargs)
+        self.WILDCARD_CERTID = self.encode_cert_id_key(ret_wildcard_hkey())
 
     def encode_cert_id_key(self, hkey):
         issuer_name_hash, issuer_key_hash, serial_number = hkey
@@ -349,9 +354,44 @@ class SnowflakeOCSPPyasn1(SnowflakeOCSP):
     def _convert_generalized_time_to_datetime(self, gentime):
         return datetime.strptime(str(gentime), '%Y%m%d%H%M%SZ')
 
-    def process_ocsp_response(self, issuer, cert_id, ocsp_response):
+    def is_valid_time(self, cert_id, ocsp_response):
         res = der_decoder.decode(ocsp_response, OCSPResponse())[0]
 
+        if res.getComponentByName('responseStatus') != OCSPResponseStatus(
+                'successful'):
+            raise OperationalError(
+                msg="Invalid Status: {0}".format(
+                    res.getComponentByName('response_status')),
+                errno=ER_INVALID_OCSP_RESPONSE)
+
+        response_bytes = res.getComponentByName('responseBytes')
+        basic_ocsp_response = der_decoder.decode(
+            response_bytes.getComponentByName('response'),
+            BasicOCSPResponse())[0]
+
+        tbs_response_data = basic_ocsp_response.getComponentByName(
+            'tbsResponseData')
+
+        single_response = tbs_response_data.getComponentByName('responses')[0]
+        cert_status = single_response.getComponentByName('certStatus')
+        try:
+            if cert_status.getName() == 'good':
+                self._process_good_status(single_response, cert_id, ocsp_response)
+        except Exception as ex:
+            logger.debug("Failed to validate ocsp response %s", ex)
+            return False
+
+        return True
+
+    def process_ocsp_response(self, issuer, cert_id, ocsp_response):
+        try:
+            res = der_decoder.decode(ocsp_response, OCSPResponse())[0]
+        except Exception:
+            raise OperationalError(
+                msg='Invalid OCSP Response',
+                errno=ER_INVALID_OCSP_RESPONSE
+            )
+
         if res.getComponentByName('responseStatus') != OCSPResponseStatus(
                 'successful'):
             raise OperationalError(
@@ -397,6 +437,7 @@ class SnowflakeOCSPPyasn1(SnowflakeOCSP):
         cert_status = single_response.getComponentByName('certStatus')
         if cert_status.getName() == 'good':
             self._process_good_status(single_response, cert_id, ocsp_response)
+            SnowflakeOCSP.OCSP_CACHE.update_cache(self, cert_id, ocsp_response)
         elif cert_status.getName() == 'revoked':
             self._process_revoked_status(single_response, cert_id)
         elif cert_status.getName() == 'unknown':
@@ -405,7 +446,7 @@ class SnowflakeOCSPPyasn1(SnowflakeOCSP):
             raise OperationalError(
                 msg="Unknown revocation status was returned. OCSP response "
                     "may be malformed: {0}".format(cert_status),
-                errno=ER_INVALID_OCSP_RESPONSE
+                errno=ER_INVALID_OCSP_RESPONSE_CODE
             )
 
     def verify_signature(self, signature_algorithm, signature, cert, data):
