@@ -32,6 +32,8 @@ setup(
     download_url='https://www.snowflake.net/',
     use_2to3=False,
 
+    python_requires='>=2.7.9,!=3.0.*,!=3.1.*,!=3.2.*,!=3.3.*',
+
     install_requires=[
         'azure-common',
         'azure-storage-blob',
@@ -46,10 +48,11 @@ setup(
         'cffi>=1.9',
         'cryptography>=1.8.2',
         'ijson',
-        'keyring!=16.1.0',
         'pyjwt',
-        'pyasn1>=0.2.1,<0.5.0',  # added back for Python 2
-        'pyasn1-modules>=0.0.8,<0.3.0',  # added back for Python 2
+        'idna',
+        'pyasn1>=0.4.0,<0.5.0;python_version<"3.0"',
+        'pyasn1-modules>=0.2.0,<0.3.0;python_version<"3.0"',
+        'enum34;python_version<"3.4"',
     ],
 
     namespace_packages=['snowflake'],
@@ -77,6 +80,12 @@ setup(
             'snowflake.connector.tool.export_certs:main',
         ],
     },
+    extras_require={
+        "secure-local-storage": [
+            'keyring!=16.1.0'
+        ],
+    },
+
     classifiers=[
         'Development Status :: 5 - Production/Stable',
 
@@ -97,6 +106,7 @@ setup(
         'Programming Language :: Python :: 3.4',
         'Programming Language :: Python :: 3.5',
         'Programming Language :: Python :: 3.6',
+        'Programming Language :: Python :: 3.7',
 
         'Topic :: Database',
         'Topic :: Software Development',
