@@ -3,8 +3,13 @@
 #
 # Copyright (c) 2018 Snowflake Computing Inc. All right reserved.
 #
+import random
 import time
 
+from logging import getLogger
+
+logger = getLogger(__name__)
+
 try:
     from threading import _Timer as Timer
 except ImportError:
@@ -27,7 +32,10 @@ class HeartBeatTimer(Timer):
         while not self.finished.is_set():
             self.finished.wait(self.interval)
             if not self.finished.is_set():
-                self.function()
+                try:
+                    self.function()
+                except Exception as e:
+                    logger.debug('failed to heartbeat: %s', e)
 
 
 def get_time_millis():
@@ -35,3 +43,14 @@ def get_time_millis():
     Return the current time in millis
     """
     return int(time.time() * 1000)
+
+
+class DecorrelateJitterBackoff(object):
+    # Decorrelate Jitter backoff
+    # https://www.awsarchitectureblog.com/2015/03/backoff.html
+    def __init__(self, base, cap):
+        self._base = base
+        self._cap = cap
+
+    def next_sleep(self, _, sleep):
+        return min(self._cap, random.randint(self._base, sleep * 3))
